import 'dart:async';

import 'package:flutter/material.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';

/// Rectangular ads that appear at the top or bottom of the device screen.
/// Banner ads stay on screen while users are interacting with the app,
/// and can refresh automatically after a certain period of time.
///
class BannerAdMod {
  Widget createBannerWidget(String bannerId,
      {AdSize bannerSize = AdSize.fullBanner}) {
    return PublisherBannerAdWidget(
      bannerId,
      bannerSize,
    );
  }
}

class PublisherBannerAdWidget extends StatefulWidget {
  PublisherBannerAdWidget(this.adId, this.size);

  final String adId;
  final AdSize size;

  @override
  State<StatefulWidget> createState() => PublisherBannerAdState();
}

class PublisherBannerAdState extends State<PublisherBannerAdWidget> {
  PublisherBannerAd _bannerAd;
  final Completer<PublisherBannerAd> bannerCompleter =
  Completer<PublisherBannerAd>();

  @override
  void initState() {
    super.initState();
    _bannerAd = PublisherBannerAd(
      adUnitId: widget.adId,
      request: PublisherAdRequest(nonPersonalizedAds: true),
      sizes: <AdSize>[widget.size],
      listener: AdListener(
        onAdLoaded: (Ad ad) {
          print('$PublisherBannerAd loaded.');
          bannerCompleter.complete(ad as PublisherBannerAd);
        },
        onAdFailedToLoad: (Ad ad, LoadAdError error) {
          ad.dispose();
          print('$PublisherBannerAd failedToLoad: $error');
          bannerCompleter.completeError(error);
        },
        onAdOpened: (Ad ad) => print('$PublisherBannerAd onAdOpened.'),
        onAdClosed: (Ad ad) => print('$PublisherBannerAd onAdClosed.'),
        onApplicationExit: (Ad ad) =>
            print('$PublisherBannerAd onApplicationExit.'),
      ),
    );
    _bannerAd.load();
  }

  @override
  void dispose() {
    super.dispose();
    _bannerAd.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder <PublisherBannerAd>(
        future: bannerCompleter.future,
        builder:
            (BuildContext context, AsyncSnapshot<PublisherBannerAd> snapshot) {
          switch (snapshot.connectionState) {
            case ConnectionState.none:
            case ConnectionState.waiting:
            case ConnectionState.active:
              return Container();
            case ConnectionState.done:
              if (snapshot.hasData) {
                return Container(
                  width: _bannerAd.sizes[0].width.toDouble(),
                  height: _bannerAd.sizes[0].height.toDouble(),
                  child: AdWidget(ad: _bannerAd),
                );
              } else
                return Container();
              break;
            default:
              return Container();
          }
        }
    );
  }
}
