import 'dart:async';

import 'package:flutter/material.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';

const String NATIVE_APP_FACTORY = "NATIVE_APP_FACTORY";

class NativeAdMod {
  Widget createNativeWidget(String adId, double width, double height) {
    return NativeAdWidget(
      adId: adId,
      width: width,
      height: height,
    );
  }
}

class NativeAdWidget extends StatefulWidget {
  final String adId;
  final double width;
  final double height;

  NativeAdWidget({Key key, this.adId, this.width, this.height})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => NativeAdState();
}

class NativeAdState extends State<NativeAdWidget> {
  NativeAd _nativeAd;
  final Completer<NativeAd> nativeAdCompleter = Completer<NativeAd>();

  @override
  void initState() {
    super.initState();
    print('app id ${widget.adId}');
    _nativeAd = NativeAd(
      adUnitId: widget.adId,
      request: AdRequest(),
      factoryId: NATIVE_APP_FACTORY,
      listener: AdListener(
        onAdLoaded: (Ad ad) {
          print('$NativeAd loaded.');
          nativeAdCompleter.complete(ad as NativeAd);
        },
        onAdFailedToLoad: (Ad ad, LoadAdError error) {
          ad.dispose();
          print('$NativeAd failedToLoad: $error');
          nativeAdCompleter.completeError(error);
        },
        onAdOpened: (Ad ad) => print('$NativeAd onAdOpened.'),
        onAdClosed: (Ad ad) => print('$NativeAd onAdClosed.'),
        onApplicationExit: (Ad ad) => print('$NativeAd onApplicationExit.'),
      ),
    );
    _nativeAd.load();
  }

  @override
  void dispose() {
    super.dispose();
    _nativeAd.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<NativeAd>(
      future: nativeAdCompleter.future,
      builder: (BuildContext context, AsyncSnapshot<NativeAd> snapshot) {
        Widget child;
        switch (snapshot.connectionState) {
          case ConnectionState.none:
          case ConnectionState.waiting:
          case ConnectionState.active:
            child = Container();
            break;
          case ConnectionState.done:
            if (snapshot.hasData) {
              child = AdWidget(ad: _nativeAd);
            } else {
              child = Container();
            }
        }

        return Container(
          width: widget.width,
          height: widget.height,
          color: Colors.grey,
          child: child,
        );
      },
    );
  }
}
